package main

import (
	"C"
)
import (
	"errors"
	"fmt"
	"net/http"
	"time"

	"github.com/PuerkitoBio/goquery"
)

func getFeedList() []Feed {
	var ret []Feed
	ret = append(ret, *NewFeed("dilbert", "Dilbert", "Scott Adams", "dilbert"))
	return ret
}

func getRssItem(slug string) (string, error) {
	desc, err := getFeedDesc(time.Now())
	if err != nil {
		return "", err
	}
	feed, err := m.GetFeed(SRC_DILBERT, slug)
	if err != nil {
		return "", err
	}
	desc = "<![CDATA[" + desc + "]]>"
	ret := "    <item>\n"
	ret += "      <title>" + feed.Name + "</title>\n"
	ret += "      <pubDate>" + feed.LastUpdate.Format(time.RFC1123Z) + "</pubDate>\n"
	ret += "      <guid>dilbert;" + slug + ";" + feed.LastUpdate.Format(time.RFC1123Z) + "</guid>\n"
	ret += "      <link>" + getFeedUrl(time.Now()) + "</link>\n"
	ret += "      <description>" + desc + "</description>\n"
	ret += "    </item>\n"
	return ret, nil
}

func getFeedUrl(date time.Time) string {
	return fmt.Sprintf(
		"http://dilbert.com/strip/%4d-%02d-%02d",
		date.Year(),
		date.Month(),
		date.Day(),
	)
}

func getFeedDesc(date time.Time) (string, error) {
	res, err := http.Get(getFeedUrl(date))
	if err != nil {
		return "", err
	}
	defer res.Body.Close()
	if res.StatusCode != 200 {
		return "", errors.New(fmt.Sprintf("Status code error: %d %s", res.StatusCode, res.Status))
	}

	// Load the HTML document
	doc, err := goquery.NewDocumentFromReader(res.Body)
	if err != nil {
		return "", err
	}
	// Find the Picture
	src, exists := doc.Find("img.img-comic").Attr("src")
	if !exists {
		return "", errors.New("Couldn't find image source")
	}
	return "<img src=\"https:" + src + "\" />", nil
}
